/*
* Global variables for C plotting package
*
* device  :  Graphics device number
* termin  :  Zero for noninteractive device, non-zero for terminal
* graphx  :  Zero if currently in text mode, non-zero for graphics mode
* currx   :  Physical x-coordinate of current point
* curry   :  Physical y-coordinate of current point
* vpp...  :  Viewport boundaries in physical coordinates
* spp...  :  Subpage  boundaries in physical coordinates
* clp...  :  Clip     boundaries in physical coordinates
* phy...  :  Physical device limits in physical coordinates
* nsub.   :  Number of subpages on physical device
* cursub  :  Current subpage
* um.     :  Number of micrometres in a pixel
* font    :  Current default font
* colour  :  Current default colour
* width   :  Current pen width
* mark    :  Array of mark lengths in micrometres for broken lines
* space   :  Array of space lengths in micrometres for broken lines
* nms     :  Number of elements for current broken line style
* timecnt :  Timer for broken lines
* alarm   :  Alarm indicating change of broken line status
* pendn   :  Flag indicating if pen is up or down
* curel   :  Current element within broken line
* inclin  :  Array of inclinations in tenths of degree for fill lines
* delta   :  Array of spacings in micrometers between fill lines
* nps     :  Number of distinct line styles for fills
* level   :  Initialization level
*
* spd...  :  Subpage  boundaries in normalized device coordinates
* vpd...  :  Viewport boundaries in normalized device coordinates
* vpw...  :  Viewport boundaries in world coordinates
* .pmm    :  Number of pixels to a millimetre
* wp....  :  Transformation variables for world  to physical conversion
* dp....  :  Transformation variables for device to physical conversion
* mp....  :  Transformation variables for millimetres from bottom left
*             hand corner to physical coordinates
* wm....  :  Transformation variables for world coordinates to mm
* chr...  :  Character default height and current (scaled) height
* sym...  :  Symbol    default height and current (scaled) height
* maj...  :  Major tick default height and current (scaled) height
* min...  :  Minor tick default height and current (scaled) height
* base3.  :  World coordinate size of base for 3-d plot
* basec.  :  Position of centre of base for 3-d plot
* dom...  :  Minimum and maximum values for domain
* zzscl   :  Vertical (z) scale for 3-d plot
* ran..   :  Minimum and maximum z values for 3-d plot
* c..     :  Coordinate transformation from 3-d to 2-d
*/

#include "plplot.h"

PLINT device, termin, graphx;
PLINT currx,  curry;
PLINT vppxmi, vppxma, vppymi, vppyma;
PLINT sppxmi, sppxma, sppymi, sppyma;
PLINT clpxmi, clpxma, clpymi, clpyma;
PLINT phyxmi, phyxma, phyymi, phyyma;
PLINT nsubx,  nsuby, cursub;
PLINT umx,    umy;
PLINT font,   colour, width, style;
PLINT mark[10], space[10], nms, timecnt;
PLINT alarm,  pendn,  curel;
PLINT inclin[2], delta[2], nps;
PLINT level=0;

PLFLT spdxmi, spdxma, spdymi, spdyma;
PLFLT vpdxmi, vpdxma, vpdymi, vpdyma;
PLFLT vpwxmi, vpwxma, vpwymi, vpwyma;
PLFLT xpmm,   ypmm;
PLFLT wpxscl, wpxoff, wpyscl, wpyoff;
PLFLT dpxscl, dpxoff, dpyscl, dpyoff;
PLFLT mpxscl, mpxoff, mpyscl, mpyoff;
PLFLT wmxscl, wmxoff, wmyscl, wmyoff;
PLFLT chrdef, chrht;
PLFLT symdef, symht;
PLFLT majdef, majht;
PLFLT mindef, minht;
PLFLT base3x, base3y, basecx, basecy;
PLFLT domxmi, domxma, domymi, domyma;
PLFLT zzscl,  ranmi,  ranma;
PLFLT cxx,    cxy,    cyx,    cyy,   cyz;

